import { ErrorService } from '@/services/error-service.js';
import usergroupsAPI from '@/api/usergroupsAPI';
import axios from 'axios';

const DEV_AUTH = process.env.NODE_ENV === 'development' ? true : false;
const AUTH = {
  username: process.env.VUE_APP_LOGIN_API_USERNAME,
  password: process.env.VUE_APP_LOGIN_API_PASSWORD
};
const path = require('path');
const DOMAIN = process.env.VUE_APP_DOMAIN;
const USERGROUP_API_PATH = process.env.VUE_APP_USERGROUP_API_PATH;

/**************** STATE *******************/
const state = {
  usergroupsRoles: [],
  spheresCount: 0,
  spheres: [],
  groupsCount: 0,
  groups: [],
  usergroupsError: null,
  isSpheresSearched: false
};

/**************** GETTERS *****************/
const getters = {
};

/*************** MUTATIONS ****************/
const mutations = {

  SET_USERGROUPS_ROLES: (state, payload) => {
    state.usergroupsRoles = payload;
  },

  SET_USERGROUPS_SPHERES_LIST: (state, payload) => {
    if (payload) {
      state.spheresCount = payload.count;
      state.spheres = payload.results;
    }
  },

  SET_USERGROUPS_GROUPS_LIST: (state, payload) => {
    if (payload) {
      state.groupsCount = payload.count;
      state.groups = payload.results;
    }
  },

  SET_ERROR: (state, error) => {
    if (error) {
      ErrorService.onError(error);
      state.usergroupsError = error;
    } else {
      state.usergroupsError = error;
    }
  },

  SET_IS_SPHERES_SEARCHED: (state, payload) => {
    state.isSpheresSearched = payload;
  }

};
/**************** ACTIONS *****************/
const actions = {

  GET_USERGROUPS_ROLES: async ({ commit }) => {
    const roles = await usergroupsAPI.getUsergroupsRoles();
    commit('SET_USERGROUPS_ROLES', roles);
  },

  GET_USERGROUP_MEMBERS: async ({}, id) => {
    try {
      const members = await usergroupsAPI.getUsergroupMembers(id);
      if (members && members.results) {
        return members.results;
      }
    } catch (err) {
      throw err;
    }
  },

  ADD_USER_TO_USERGROUP: async ({}, { usergroup, data }) => {
    try {
      const response = await usergroupsAPI.addUserToUsergroup(usergroup.usergroup.id, data);
      if (response) {
        ErrorService.onSuccess(response, `Vous avez bien été ajouté à l\'organisation ${usergroup.display_name}`);
      }
    } catch (err) {
      throw err;
    }
  },
  REMOVE_USER_FROM_USERGROUP: async ({}, { usergroup, userId}) => {
    try {
      const response = await usergroupsAPI.removeUserFromUsergroup(usergroup.usergroup.id, userId);
      if (response) {
        ErrorService.onSuccess(response, `Vous avez bien été retiré de l\'organisation ${usergroup.display_name}`);
      }
    } catch (err) {
      throw err;
    }
  },

  GET_GROUPS_LIST: async ({ commit }) => {
    try {
      const groups = await usergroupsAPI.getFilteredUsergroupsList('user-group');
      if (groups) {
        commit('SET_ERROR', null);
        commit('SET_USERGROUPS_GROUPS_LIST', groups);
      }
    } catch (error) {
      commit('SET_ERROR', error);
    }
  },

  GET_SPHERES_LIST: async ({ commit }) => {
    try {
      const spheres = await usergroupsAPI.getFilteredUsergroupsList()
      if (spheres) {
        commit('SET_ERROR', null);
        commit('SET_USERGROUPS_SPHERES_LIST', spheres);
      }
    } catch (error) {
      commit('SET_ERROR', error);
    }
  },

  SEARCH_SPHERES: async ({ rootState, commit }, text) => {
    if (rootState.cancellableSearchRequest) {
      rootState.cancellableSearchRequest.cancel();
      commit('SET_CANCELLABLE_SEARCH_REQUEST', null, { root: true });
    }

    const cancelToken = axios.CancelToken.source();
    commit('SET_CANCELLABLE_SEARCH_REQUEST', cancelToken, { root: true });
    const url = new URL(path.join(USERGROUP_API_PATH, `user-groups/?page=1&search=${text}&usergroup_types=group-of-organisation`), DOMAIN);

    try {
      const response = await axios.get(
        url,
        {
          cancelToken: cancelToken.token,
          ...DEV_AUTH && { auth: AUTH }
        }
      );
      if (response.status === 200) {
        commit('SET_CANCELLABLE_SEARCH_REQUEST', null, { root: true });
        const usergroups = response.data;
        if (usergroups) {
          commit('SET_ERROR', null);
          commit('SET_USERGROUPS_SPHERES_LIST', usergroups);
          commit('SET_IS_SPHERES_SEARCHED', true);
        }
      }
    } catch(err) {
      commit('SET_CANCELLABLE_SEARCH_REQUEST', null, { root: true });
    }
  }
};

export default {
  namespaced: true,
  state,
  getters,
  actions,
  mutations,
};
